class Pathfinder(object):
    def run(self, start, goal, *args, **kwargs):
        start = tuple(start)
        open = set([start])
        closed = set()
        g_score = {start: 0}
        f_score = {start: g_score[start]+self.remaining(start, goal, *args, **kwargs)}
        came_from = {}
        while open:
            x = min(open, key=lambda x: f_score[x])
            if x == goal:
                res = [goal]
                while not res[-1] == start:
                    res.append(came_from[res[-1]])
                res.reverse()
                return res
            open.remove(x)
            closed.add(x)
            for y in self.children(x, *args, **kwargs):
                y = tuple(y)
                if y in closed:
                    continue
                tentative_g_score = g_score[x] + self.cost(x,y, *args, **kwargs)
                tentative_is_better = False
                if y not in open:
                    open.add(y)
                    h_score = self.remaining(y, goal, *args, **kwargs)
                    tentative_is_better = True
                elif tentative_g_score < g_score[y]:
                    tentative_is_better = True
                if tentative_is_better:
                    came_from[y] = x
                    g_score[y] = tentative_g_score
                    f_score[y] = g_score[y] + h_score
    
    def children(self, node, *args, **kwargs):
        "Return list of nodes"
        raise NotImplementedError
    
    def cost(self, src, dst, *args, **kwargs):
        "Return integer"
        raise NotImplementedError
    
    def remaining(self, node, dest, *args, **kwargs):
        "Return integer"
        raise NotImplementedError

class LinearPathfinder(Pathfinder):
    def cost(self, src, dst, *args, **kwargs):
        return ((src[0]-dst[0])**2+(src[1]-dst[1])**2)**.5
    remaining = cost

class TestPathfinder(LinearPathfinder):
        def children(self, n, nodes, dist):
            return [x for x in nodes if self.cost(n, x) < dist]

if __name__ == "__main__":
    import pygame
    import random
    import time
    while True:
        nodes = [tuple([random.randrange(500) for j in xrange(2)]) for i in xrange(100)]
        start = random.choice(nodes)
        end = random.choice(nodes)
        d = pygame.display.set_mode((500, 500))
        for n in nodes:
            pygame.draw.circle(d, (255, 255, 255) if n not in (start, end) else (255, 0, 0), n, 3)
        pygame.display.update()
        p = TestPathfinder()
        res = p.run(start, end, nodes, 100)
        if not res: continue
        o = None
        for i in res:
            if o:
                pygame.draw.aaline(d, (0, 0, 255), o, i)
            o = i
        for n in nodes:
            pygame.draw.circle(d, (255, 255, 255) if n not in (start, end) else (255, 0, 0), n, 3)
        pygame.display.update()
        time.sleep(1.5)
